#pragma once
#include "opencv2/opencv.hpp"
#include "Feature.h"
#include "ImageStack.h"
#include "Tracker.h"

//Energy Evaluation functions for Multi-trackers - No gyro terms  *************************************************************************************************************
double compute_c(cv::Mat& xbar, FeatureSet& features, ImageStack& frameStack, int resolutionLevel, cv::Mat& History, MultiTrackerParameters *TrackerParams);
void   compute_c(cv::Mat& xbar, FeatureSet& features, ImageStack& frameStack, int resolutionLevel, cv::Mat& History, double& c, cv::Mat& Dc, MultiTrackerParameters *TrackerParams);

//Energy Evaluation functions for Multi-trackers - With gyro terms  ***********************************************************************************************************
double compute_c_withPrior(cv::Mat& xbar, cv::Mat& xbar_prior, FeatureSet& features, ImageStack& frameStack, int resolutionLevel, cv::Mat& History, MultiTrackerParameters *TrackerParams, double lambda);
void   compute_c_withPrior(cv::Mat& xbar, cv::Mat& xbar_prior, FeatureSet& features, ImageStack& frameStack, int resolutionLevel, cv::Mat& History, double& c, cv::Mat& Dc, MultiTrackerParameters *TrackerParams, double lambda);

double compute_fitTerm     (ImageStack& Template, ImageStack& frameStack, int resolutionLevel, cv::Vec2f location, int n);
void   compute_fitTerm     (ImageStack& Template, ImageStack& frameStack, int resolutionLevel, cv::Vec2f location, int n, double& residual, double& Dx, double& Dy);

//Penalty term evaluation functions. compute_P looks into TrackerParams to find the penalty function form and calls the appropriate compute_P_* function.
double compute_P(cv::Mat& xbar, cv::Mat& History, MultiTrackerParameters *TrackerParams);
void   compute_P(cv::Mat& xbar, cv::Mat& History, double& P, cv::Mat& DP, MultiTrackerParameters *TrackerParams);

double compute_P_EmpDim(cv::Mat& xbar, cv::Mat& History, MultiTrackerParameters *TrackerParams);
void   compute_P_EmpDim(cv::Mat& xbar, cv::Mat& History, double& P, cv::Mat& DP, MultiTrackerParameters *TrackerParams);

double compute_P_NucNorm(cv::Mat& xbar, cv::Mat& History, MultiTrackerParameters *TrackerParams);
void   compute_P_NucNorm(cv::Mat& xbar, cv::Mat& History, double& P, cv::Mat& DP, MultiTrackerParameters *TrackerParams);

double compute_P_ExpFact(cv::Mat& xbar, cv::Mat& History, MultiTrackerParameters *TrackerParams);
void   compute_P_ExpFact(cv::Mat& xbar, cv::Mat& History, double& P, cv::Mat& DP, MultiTrackerParameters *TrackerParams);

//Prior regularization term evaluation functions - for multi-feature trackers
double compute_priorTerm(cv::Mat& xbar, cv::Mat& xbar_prior, double lambda);
void compute_priorTerm(cv::Mat& xbar, cv::Mat& xbar_prior, double lambda, double& E, cv::Mat& DE);
