#include "ImageStack.h"

using namespace cv;
using namespace std;

int ImageStack::levels(void) const {
	return(Images.size());
}

//TODO: Speedup here by eliminating unnecessary copy from currentLevel to Images[m] on all but level 0.
void ImageStack::BuildFromImage(const Mat& SourceImage, int levels) {
	Images.resize(levels);
	Mat lastLevel, currentLevel;
	for (int m=0; m<levels; m++) {
		if (m == 0)
			currentLevel = SourceImage;
		else {
			lastLevel = currentLevel;
			pyrDown(lastLevel, currentLevel);
		}
		Images[m] = Mat(currentLevel.rows, currentLevel.cols, currentLevel.type()); //Explicitly allocate image space
		
		currentLevel.copyTo(Images[m]);
	}
}

void ImageStack::BuildFromStack(const ImageStack& sourceStack, const Vec2f& location, int width, int height) {
	Vec2f locationThisLevel;
	Images.resize(sourceStack.levels());
	if (height == -1)
		height = width;
	for (int m=0; m<((int) Images.size()); m++) {
		convertImageCoordinates(location, locationThisLevel, 0, m);
		if ((locationThisLevel[0] >= 0) && (locationThisLevel[0] < sourceStack.Images[m].rows) && (locationThisLevel[1] >= 0) && (locationThisLevel[1] < sourceStack.Images[m].cols)) {
			getRectSubPix(sourceStack.Images[m], Size(width,height), Point2f(locationThisLevel[1], locationThisLevel[0]), Images[m]);
		}
		else
			Images[m] = Mat::zeros(height, width, sourceStack.Images[0].type());
	}
}

void ImageStack::convertImageCoordinates(const Vec2f& inputCoords, Vec2f& outputCoords, int inputLevel, int outputLevel) {
	if (inputLevel == outputLevel) {
		outputCoords = inputCoords;
	}
	else if (outputLevel > inputLevel) {
		//We are dropping in resolution
		float n = (float)(outputLevel-inputLevel);
		float temp = pow(2, n);
		//outputCoords[0] = inputCoords[0]/temp + (temp - 1)/temp;
		//outputCoords[1] = inputCoords[1]/temp + (temp - 1)/temp;
		
		outputCoords[0] = inputCoords[0]/temp;
		outputCoords[1] = inputCoords[1]/temp;
	}
	else { //outputLevel < inputLevel
		//We are increasing in resolution
		float n = (float)(inputLevel-outputLevel);
		float temp = pow(2, n);
		//outputCoords[0] = temp*inputCoords[0] - (temp - 1);
		//outputCoords[1] = temp*inputCoords[1] - (temp - 1);
		
		outputCoords[0] = temp*inputCoords[0];
		outputCoords[1] = temp*inputCoords[1];
	}
}
